# 
# 	Sebastien.Letort@toulouse.inra.fr
#	Created: April 05, 2007
#	Last Updated: April 16, 2007
#
# Copyright INRA

# Sebastien.Carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide
# a code generator framework for BioMoby web-services.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

package General;

BEGIN {
	use Exporter   ();
	use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

	# Si vous utilisez RCS/CVS, ceci serait preferable
	# le tout sur une seule ligne, pour MakeMaker
	$VERSION = do { my @r = (q$Rev: 86 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };

	@ISA         = qw(Exporter);
	@EXPORT      = qw(&TRUE &FALSE IsDefined SetUnlessDefined);
	%EXPORT_TAGS = ( );     # ex. : TAG => [ qw!name1 name2! ],

	# vos variables globales a etre exporter vont ici,
	# ainsi que vos fonctions, si necessaire
	@EXPORT_OK   = qw(&ShowDebug &GetDirectoryFile &SetUnlessDefined);
}

use Data::Dumper;

# les constantes
use constant TRUE => 1;
use constant FALSE => 0;

#les fonctions generales

=head2 Procedure ShowDebug

	Title      :	ShowDebug
	Usage      :	ShowDebug( {deep=>$deep}, @a_args );
	Prerequiste:	none
	Procedure  :	print onto STDERR the name of the function where the function was called
	        	depending on the deep parameter
	        	with the filename and the line number of the current file.
	        	Then it prints the list of arguments with Dumper.
	Args       :	$deep, an integer representing the deep of the call
	        		must be >=1
	        	@a_args, the list of arguments to print out.
	Error      :    none
	Globals    :    none
	
	Note       :	I used a hash ref to preserved compatibility with previous usage ShowDebug(@a_args)

=cut

sub ShowDebug
{
	my (@args) = @_;

	my $deep = 1;
	if( ref $args[0] eq 'HASH' )
	{
		$rh_hash = shift @args;
		$deep = $$rh_hash{ deep };
	}

	my ($package, $file, $line) = caller();

	my $subroutine = ( caller($deep) )[3] || $package;
	print STDERR "In $subroutine ($file:$line) :\n", Data::Dumper->Dump([@args]);
}


=head2 Function IsDefined

	Title      :	IsDefined
	Usage      :	print "\$var n'est pas vide\n"	if( IsDefined( $var ) );
	Prerequiste:	only works with string, any ideas to enhaced with interger (!=0) is welcome
	Function   :	shortcut to know if a variable is defined and non empty
	Returns    :	1 if var is defined and non empty, 0 otherwise
	Args       :	$var, a variable that should contains a string.
	Error      :    none
	Globals    :    none

=cut

sub IsDefined
{
	my ($var) = @_;

	return( defined $var and $var ne '');
}

=head2 Function IsDefined

	Title      :	IsDefined
	Usage      :	print "\$var n'est pas vide\n"	if( IsDefined( $var ) );
	Prerequiste:	only works with string, any ideas to enhaced with interger (!=0) is welcome
	Function   :	shortcut to know if a variable is defined and non empty
	Returns    :	1 if var is defined and non empty, 0 otherwise
	Args       :	$var, a variable that should contains a string.
	Error      :    none
	Globals    :    none

=cut

sub GetDirectoryFile
{
	my ($file) = @_;
	
	my $abs_path = $INC{$file};
	use File::Basename;

	return( dirname($abs_path) );
}

=head2 Function SetUnlessDefined

	Title      :	SetUnlessDefined
	Usage      :	SetUnlessDefined( \$var, $val );	# prefer this one
	            	$var = SetUnlessDefined( $var, $val );
	Prerequiste:	none
	Function   :	give a value to a variable if it hasn't got any.
	Returns    :	the value of the variable if a scalar is given as the first arg.
	            	nothing otherwise.
	Args       :	$var, a variable.
	Error      :    none
	Globals    :    none

=cut

sub SetUnlessDefined
{
	my ( $r_var, $val ) = @_;

	if( ref( $r_var ) eq 'SCALAR' )
	{
		$$r_var = $val	unless IsDefined( $$r_var );
		return;
	}
	elsif( ref( $r_var ) eq '' )
	{
		return  $val	unless IsDefined( $r_var );
		return $r_var;
	}

	return;
}


1;
